@@ -1,13 +1,14 @@
 import inspect
-from pprint import pprint
 import pandas as pd
 import copy
-import LibHanger.Library.uwLogger as Logger
-from sqlalchemy.exc import SQLAlchemyError
-from sqlalchemy import and_
 from typing import TypeVar, Generic
 from enum import Enum
+from sqlalchemy.exc import SQLAlchemyError
+from sqlalchemy import and_
+from sqlalchemy.orm.session import Session as BaseSession
+import LibHanger.Library.uwLogger as Logger
 from LibHanger.Models.fields import fields
+from LibHanger.Library.DataAccess.Base.uwDataAccess import uwDataAccess
 
 T = TypeVar('T')
 
@@ -20,6 +21,9 @@ class recset(Generic[T]):
     __ROWSTATE_COLNAME__ = '__rowState'
     """ 行状態列名 """
 
+    __SA_INSTANCE_STATE__ = '_sa_instance_state'
+    """ _sa_instance_state """
+    
     class rowState(Enum):
 
         """
@@ -38,7 +42,7 @@ class recset(Generic[T]):
         deleted = 3
         """ 削除 """
         
-    def __init__(self, t, __session = None, __where = None) -> None:
+    def __init__(self, t, __da:uwDataAccess = None, __where = None) -> None:
         
         """
         コンストラクタ
@@ -47,8 +51,8 @@ class recset(Generic[T]):
         ----------
         t : Any
             Modelクラス
-        __session : Any
-            DBセッション
+        __da : Any
+            データアクセスクラスインスタンス
         __where : Any
             Where条件
         """
@@ -56,8 +60,13 @@ class recset(Generic[T]):
         # Modelクラス
         self.modelType = t
 
-        # DBセッション保持
-        self.__session = __session
+        # DataAccessクラス保持
+        self.__da = __da
+        
+        # session
+        self.__session = None
+        if not self.__da is None:
+            self.__session = self.__da.session
         
         # カラム情報
         self.__columns = self.__getColumnAttr()
@@ -137,7 +146,7 @@ class recset(Generic[T]):
     def State(self) ->int:
         
         """
-        行状態
+        行状態(getter)
         """
         
         return self.__rows[self.__currentRowIndex].__rowState
@@ -145,6 +154,10 @@ class recset(Generic[T]):
     @State.setter
     def State(self, __rowState):
         
+        """
+        行状態(setter)
+        """
+        
         # rowsのカレント行の行状態をmodifiedに変更
         if self.__rows[self.__currentRowIndex].__rowState == self.rowState.noChanged:
             self.__rows[self.__currentRowIndex].__rowState = __rowState
@@ -229,10 +242,10 @@ class recset(Generic[T]):
         # 主キー情報を返す
         return primaryKeys
     
-    def __getPKeyFilter(self, row):
+    def __getPKeyFilter_sa(self, row):
         
         """
-        主キー条件取得
+        主キー条件取得(SQL Alchemy用)
         
         row : Any
             行情報
@@ -256,7 +269,7 @@ class recset(Generic[T]):
         
         Parameters
         ----------
-        row : any
+        row : Any
             行情報
         """
         
@@ -298,6 +311,14 @@ class recset(Generic[T]):
         
         """
         対象行をレコードソース(DataFrame)に追加する
+        
+        Parameters
+        ----------
+        row : Any
+            行情報
+        targetRowIndex : int
+            DataFrame行インデックス
+
         """
         
         # 行情報コピー
@@ -337,24 +358,56 @@ class recset(Generic[T]):
         
         """
         行情報をコピーする
+        
+        Parameters
+        ----------
+        rows : list
+            行情報リスト
         """
 
         # 行インスタンスをDeepCopy
         targetRows = copy.deepcopy(rows)
         # DataFrame化で不要な列を削除
         for rowInstance in targetRows:
-            delattr(rowInstance, '_sa_instance_state')
+            delattr(rowInstance, self.__SA_INSTANCE_STATE__)
             
         return targetRows
 
-    def setSession(self, __session):
+    def __getKeyTuple(self, row):
+        
+        """
+        rowのキー値をtupleで取得
+
+        Parameters
+        ----------
+        row : Any
+            行情報
+        """
+        
+        # 主キー値リスト初期化
+        pKeyValueList = []
+    
+        # 主キー値をリストに追加
+        for key in self.__getPrimaryKeys():
+            pKeyValueList.append(getattr(row, key))
+        
+        # tupleに変換(dictrowのKeyにする)
+        return tuple(pKeyValueList)
+
+    def setDA(self, __da:uwDataAccess):
 
         """
-        DBセッションセット
+        DataAccessクラスインスタンスセット
+
+        Parameters
+        ----------
+        __da : uwDataAccess
+            DataAccessクラスインスタンス
         """
 
-        if self.__session is None:
-            self.__session = __session
+        if self.__da is None:
+            self.__da = __da
+            self.__session = self.__da.session
 
     def newRow(self):
         
@@ -390,7 +443,6 @@ class recset(Generic[T]):
         レコードセットのカレント行を削除対象とする
         """
 
-
         # 行状態をdeletedに変更
         self.State = self.rowState.deleted
 
@@ -417,7 +469,8 @@ class recset(Generic[T]):
 
         Parameters
         ----------
-        None
+        __rows : list
+            行情報リスト
 
         """
         
@@ -481,20 +534,22 @@ class recset(Generic[T]):
         
         """
         対象行に対して主キー制約に違反しているか
-        
+
+        Parameters
+        ----------
         row : Any
             行情報
         """
         
         # 主キーを条件として該当レコードが存在するか確認
-        w = self.__getPKeyFilter(row)
+        w = self.__getPKeyFilter_sa(row)
         q = self.__session.query(self.modelType).filter(w).all()
         for qrow in q:
             
             dictRow = vars(row)
             dictRow_sorted = sorted(dictRow.items())
             for col in dictRow_sorted:
-                if col[0] == '_sa_instance_state': continue
+                if col[0] == self.__SA_INSTANCE_STATE__: continue
                 setattr(qrow, col[0], getattr(row, col[0]))
             
             self.__rows.append(qrow)
@@ -508,6 +563,8 @@ class recset(Generic[T]):
         """
         現在保持しているレコードセットに指定した条件に合致するレコードが存在するか返す
 
+        Parameters
+        ----------
         w : Any
             抽出条件
         """
@@ -517,6 +574,60 @@ class recset(Generic[T]):
 
         return True if len(dfCheck) > 0 else False
 
+    def merge(self, srcRecset):
+        
+        """
+        レコードセットをマージする
+
+        Parameters
+        ----------
+        srcRecset : Any
+            マージ元レコードセット
+            
+        Notes
+        -----
+            マージ先に同一のキー情報が存在した場合はマージ対象から除外
+        """
+        
+        # rows⇒dict変換
+        dictrows = dict()
+        for row in self.__rows:
+            
+            # tupleに変換(dictrowのKeyにする)
+            pKeyTuple = self.__getKeyTuple(row)
+            
+            # rowをdictrowsにセット
+            if not pKeyTuple in dictrows:
+                dictrows[pKeyTuple] = row
+            
+        # マージ対象レコードセットをrowsに追加
+        for mergeRow in srcRecset.rows:
+            
+            # rowのキー値取得
+            pKeyTuple = self.__getKeyTuple(mergeRow)
+            
+            # キー値がdictrowに無ければ__dictrowsに追加
+            if not pKeyTuple in dictrows:
+                dictrows[pKeyTuple] = mergeRow
+
+        # dictrowsをソート
+        rows_sorted = sorted(dictrows.items())
+        
+        # rows再構築
+        self.__rows.clear()
+        for row in rows_sorted:
+            self.__rows.append(row[1])
+        
+        # dfrows再構築
+        self.__dfrows = self.__dfrows.drop(range(len(self.__dfrows)))
+        rowIndex = 0
+        for row in rows_sorted:
+            self.__addRecrowDataFrame(row[1], rowIndex)
+            rowIndex += 1
+        
+        # fields再構築
+        self.__fields = fields(self.__rows, self.__dfrows)
+        
     def upsert(self):
 
         """
@@ -560,7 +671,8 @@ class recset(Generic[T]):
                 self.__session.delete(delRow)
 
             # Commit
-            self.__session.commit()
+            if self.__da.autoCommit:
+                self.__session.commit()
 
             # 処理結果セット
             result = True
@@ -571,7 +683,8 @@ class recset(Generic[T]):
             Logger.logging.error(e)
 
             # Rollback
-            self.__session.rollback()
+            if self.__da.autoCommit:
+                self.__session.rollback()
         
         # 処理結果を返す
         return result
\ No newline at end of file
